@@ -32,7 +32,7 @@
  *     this has the advantage of being slightly faster, but
  *     demands that all hints are handled at one place. WM ignores
  *     other hints of the same type if these two are present.
- 
+
  *   - check out WMaker and KDE hints to make a superset of all
  *   - virtual desktop, handling of virtual (>screen) scrolling
  *     desktops and pages (fvwm like?)
@@ -81,7 +81,7 @@
  *
  * Icon Mask can be None if transparency is not required.
  *
- * Icewm currenty needs/uses icons of size 16x16 and 32x32 with default
+ * Icewm currently needs/uses icons of size 16x16 and 32x32 with default
  * depth. Applications are recommended to set several icons sizes
  * (recommended 16x16,32x32,48x48 at least)
  *
@@ -111,7 +111,7 @@
  *     XSendEvent(display, root, False, SubstructureNotifyMask, (XEvent *) &xev);
  *
  */
- 
+
 #define XA_WIN_WORKSPACE_COUNT "_WIN_WORKSPACE_COUNT"
 /* Type: CARD32
  *       workspace count, set by window manager
@@ -122,13 +122,13 @@
 #define XA_WIN_WORKSPACE_NAMES "_WIN_WORKSPACE_NAMES"
 /* Type: StringList (TextPropery)
  *
- * 
+ *
  * IMPLEMENTED but not FINALIZED.
  * perhaps the name should be separate for each workspace (like KDE).
  * this where WIN_WORKSPACE_COUNT comes into play.
  */
 
-#define WinWorkspaceInvalid    (-1L)
+#define WinWorkspaceInvalid    -1L
 
 /* workspaces */
 #define XA_WIN_WORKSPACES "_WIN_WORKSPACES"
@@ -183,13 +183,13 @@
  * TODO: A few available layers could be used for WMaker menus/submenus
  * (comments?)
  *
- * Partially implemented. Currently requires all docked windows to be 
+ * Partially implemented. Currently requires all docked windows to be
  * sticky (only one workarea for all workspaces). Otherwise non-docked sticky
  * windows could (?) move when switching workspaces (annoying).
  */
- 
+
 #define WinLayerCount          16
-#define WinLayerInvalid        0xFFFFFFFFL
+#define WinLayerInvalid        -1L
 
 #define WinLayerDesktop        0L
 #define WinLayerBelow          2L
@@ -228,7 +228,7 @@
  */
 
 #define WinTrayOptionCount      3
-#define WinTrayInvalid          0xFFFFFFFFL
+#define WinTrayInvalid          -1L
 
 #define WinTrayIgnore           0L
 #define WinTrayMinimized        1L
@@ -248,43 +248,70 @@
  *
  *   xev.type = ClientMessage;
  *   xev.window = toplevel_window;
- *   xev.message_type = _XA_WIN_WORKSPACE;
+ *   xev.message_type = _XA_WIN_STATE;
  *   xev.format = 32;
  *   xev.data.l[0] = mask; // mask of the states to change
  *   xev.data.l[1] = state; // new state values
  *   xev.data.l[2] = timeStamp;
  *   XSendEvent(display, root, False, SubstructureNotifyMask, (XEvent *) &xev);
  */
+#if 0
+// From wm-comp.ps:
+#define WIN_STATE_STICKY            (1<<0) /* everyone knows sticky */
+#define WIN_STATE_MINIMIZED         (1<<1) /* Reserved - definition is unclear */
+#define WIN_STATE_MAXIMIZED_VERT    (1<<2) /* window in maximized V state */
+#define WIN_STATE_MAXIMIZED_HORIZ   (1<<3) /* window in maximized H state */
+#define WIN_STATE_HIDDEN            (1<<4) /* not on taskbar but window visible */
+#define WIN_STATE_SHADED            (1<<5) /* shaded (MacOS / Afterstep style) */
+#define WIN_STATE_HID_WORKSPACE     (1<<6) /* not on current desktop */
+#define WIN_STATE_HID_TRANSIENT     (1<<7) /* owner of transient is hidden */
+#define WIN_STATE_FIXED_POSITION    (1<<8) /* window is fixed in position even */
+#define WIN_STATE_ARRANGE_IGNORE    (1<<9) /* ignore for auto arranging */
+#endif
 
-#define WinStateAllWorkspaces  (1 << 0)   /* appears on all workspaces */
+#define WinStateSticky         (1 << 0)   /* sticks to virtual desktop */
 #define WinStateMinimized      (1 << 1)   /* to iconbox,taskbar,... */
 #define WinStateMaximizedVert  (1 << 2)   /* maximized vertically */
 #define WinStateMaximizedHoriz (1 << 3)   /* maximized horizontally */
+#define WinStateMaximizedBoth  (3 << 2)   /* maximized vert+horiz */
 #define WinStateHidden         (1 << 4)   /* not on taskbar if any, but still accessible */
 #define WinStateRollup         (1 << 5)   /* only titlebar visible */
-#define WinStateFixedPosition  (1 << 10)  /* fixed position on virtual desktop*/
-#define WinStateArrangeIgnore  (1 << 11)  /* ignore for auto arranging */
-//#define WinStateDocked         (1 << 9) /* docked, ignore my area for maximizing */
+#define WinStateHidWorkspace   (1 << 6)   /* not on current desktop */
+#define WinStateHidTransient   (1 << 7)   /* owner of transient is hidden */
+#define WinStateFixedPosition  (1 << 8)   /* fixed position on virtual desktop*/
+#define WinStateArrangeIgnore  (1 << 9)   /* ignore for auto arranging */
+#define WinStateFocused        (1 << 21)  /* has the focus */
+#define WinStateUrgent         (1 << 22)  /* demands attention */
+#define WinStateSkipPager      (1 << 23)  /* skip pager */
 #define WinStateSkipTaskBar    (1 << 24)  /* skip taskbar */
 #define WinStateModal          (1 << 25)  /* modal */
 #define WinStateBelow          (1 << 26)  /* below layer */
 #define WinStateAbove          (1 << 27)  /* above layer */
-#define WinStateFullscreen     (1 << 28)  /* fullscreen (no lauout limits) */
+#define WinStateFullscreen     (1 << 28)  /* fullscreen (no layout limits) */
 #define WinStateWasHidden      (1 << 29)  /* was hidden when parent was minimized/hidden */
 #define WinStateWasMinimized   (1 << 30)  /* was minimized when parent was minimized/hidden */
 #define WinStateWithdrawn      (1 << 31)  /* managed, but not available to user */
 
-#define WIN_STATE_ALL (WinStateAllWorkspaces | WinStateMinimized |\
+#define WIN_STATE_ALL (WinStateSticky | WinStateMinimized |\
                        WinStateMaximizedVert | WinStateMaximizedHoriz |\
-                       WinStateHidden | WinStateRollup)
+                       WinStateHidden | WinStateRollup | WinStateHidWorkspace |\
+                       WinStateHidTransient | WinStateFixedPosition |\
+                       WinStateArrangeIgnore)
 
 
 /* hints */
+// From wm-comp.ps:
+// #define WIN_HINTS_SKIP_FOCUS        (1<<0) /* "alt-tab" skips this win */
+// #define WIN_HINTS_SKIP_WINLIST      (1<<1) /* do not show in window list */
+// #define WIN_HINTS_SKIP_TASKBAR      (1<<2) /* do not show on taskbar */
+// #define WIN_HINTS_GROUP_TRANSIENT   (1<<3) /* Reserved - definition is unclear */
+// #define WIN_HINTS_FOCUS_ON_CLICK    (1<<4) /* app only accepts focus if clicked */
+
 #define XA_WIN_HINTS            "_WIN_HINTS"
-#define WinHintsSkipFocus       (1 << 0)
-#define WinHintsSkipWindowMenu  (1 << 1)
-#define WinHintsSkipTaskBar     (1 << 2)
-#define WinHintsGroupTransient  (1 << 3)
+#define WinHintsSkipFocus       (1 << 0) /* "alt-tab" skips this win */
+#define WinHintsSkipWindowMenu  (1 << 1) /* do not show in window list */
+#define WinHintsSkipTaskBar     (1 << 2) /* do not show on taskbar */
+#define WinHintsGroupTransient  (1 << 3) /* Reserved - definition is unclear */
 #define WinHintsFocusOnClick    (1 << 4) /* app only accepts focus when clicked */
 #define WinHintsDoNotCover      (1 << 5) /* attempt to not cover this window */
 #define WinHintsDockHorizontal  (1 << 6) /* docked horizontally */
@@ -293,33 +320,6 @@
                        WinHintsSkipTaskBar | WinHintsGroupTransient |\
                        WinHintsFocusOnClick | WinHintsDoNotCover)
 
-/* Type CARD32[2]
- *      additional window hints
- *
- *      Handling of this propery is very similiar to WIN_STATE.
- *
- * NOT IMPLEMENTED YET.
- */
-/* WinHintsDockHorizontal -- not used
- * This state is necessary for correct WORKAREA negotiation when
- * a window is positioned in a screen corner. If set, it determines how
- * the place where window is subtracted from workare.
- *
- * Imagine a square docklet in the corner of the screen (several WMaker docklets
- * are like this).
- *
- * HHHHD
- * ....V
- * ....V
- * ....V
- *
- * If WinStateDockHorizontal is set, the WORKAREA will consist of area
- * covered by '.' and 'V', otherwise the WORKAREA will consist of area
- * covered by '. and 'H';
- *
- * currently hack is used where: w>h -> horizontal dock, else vertical
- */
-
 /* work area of current workspace -- */
 #define XA_WIN_WORKAREA        "_WIN_WORKAREA"
 /*
@@ -349,4 +349,9 @@
 #define XA_WIN_AREA "_WIN_AREA"
 #define XA_WIN_AREA_COUNT "_WIN_AREA_COUNT"
 
+#define XA_WIN_APP_STATE "_WIN_APP_STATE"
+#define XA_WIN_EXPANDED_SIZE "_WIN_EXPANDED_SIZE"
+
 #endif
+
+// vim: set sw=4 ts=4 et:
