@@ -27,10 +27,10 @@ WorkspaceButton::WorkspaceButton(int ws, YWindow *parent, WorkspaceDragger* d):
     fDelta(0),
     fDownX(0),
     fDragging(false),
-    fGraphics(this),
+    fGraphics(this, true),
     fPane(d)
 {
-    setStyle(wsNoExpose);
+    addStyle(wsNoExpose);
     setParentRelative();
     //setDND(true);
     setTitle(name());
@@ -46,23 +46,6 @@ void WorkspaceButton::repaint() {
     fGraphics.paint();
 }
 
-void WorkspaceButton::paintBackground(Graphics& g, const YRect& r) {
-    if (taskbackPixbuf != null) {
-        g.drawGradient(taskbackPixbuf,
-                       r.x(), r.y(), r.width(), r.height(),
-                       0, 0, width(), height());
-    }
-    else if (taskbackPixmap != null) {
-        g.fillPixmap(taskbackPixmap,
-                     r.x(), r.y(), r.width(), r.height(),
-                     r.x(), r.y());
-    }
-    else {
-        g.setColor(taskBarBg);
-        g.fillRect(r.x(), r.y(), r.width(), r.height());
-    }
-}
-
 void WorkspaceButton::handleButton(const XButtonEvent &button) {
     if (fDragging &&
         button.type == ButtonPress &&
@@ -78,7 +61,7 @@ void WorkspaceButton::handleClick(const XButtonEvent &up, int count) {
     switch (up.button) {
         case 1:
             if (count == 2) {
-                if (fInput == 0) {
+                if (fInput == nullptr) {
                     fInput = new YInputLine(this, this);
                 }
                 if (fInput) {
@@ -173,26 +156,26 @@ bool WorkspaceButton::handleTimer(YTimer *t) {
 
 void WorkspaceButton::inputReturn(YInputLine* input) {
     if (input == fInput) {
-        cstring text(input->getText());
+        mstring text(input->getText());
         if (text != name()) {
             csmart str(newstr(text));
             swap(*&str, *workspaces[fWorkspace]);
             manager->setDesktopNames();
             fPane->relabel(fWorkspace);
         }
-        fInput = 0;
+        fInput = nullptr;
     }
 }
 
 void WorkspaceButton::inputEscape(YInputLine* input) {
     if (input == fInput) {
-        fInput = 0;
+        fInput = nullptr;
     }
 }
 
 void WorkspaceButton::inputLostFocus(YInputLine* input) {
     if (input == fInput) {
-        fInput = 0;
+        fInput = nullptr;
     }
 }
 
@@ -231,7 +214,7 @@ WorkspacesPane::WorkspacesPane(YWindow *parent):
     fRepositioning(false),
     fReconfiguring(false)
 {
-    setStyle(wsNoExpose);
+    addStyle(wsNoExpose);
     setParentRelative();
 }
 
@@ -251,16 +234,19 @@ void WorkspacesPane::resize(unsigned width, unsigned height) {
 
 long WorkspacesPane::limitWidth(long paneWidth) {
     const char* str = taskBarWorkspacesLimit;
+    long taskBarWidth = desktop->getScreenGeometry().width()
+                      * taskBarWidthPercentage / 100;
+    long reserved = 200L;
     long maxPixels = 0;
     long maxButtons = 0;
     long maxPercent = 0;
     if (nonempty(str)) {
-        char* end = 0;
+        char* end = nullptr;
         long num = strtol(str, &end, 0);
         if (end && str < end && inrange(num, 0L, long(SHRT_MAX))) {
-            maxPixels = max(50L, long(desktop->width()) - x() - 20L);
+            maxPixels = max(50L, taskBarWidth - x() - reserved);
             maxButtons = maxPixels * count() / non_zero(paneWidth);
-            maxPercent = 100L * maxPixels / desktop->width();
+            maxPercent = 100L * maxPixels / taskBarWidth;
             if (*end == ' ')
                 ++end;
             if (*end == 'p' || *end == 'P') {
@@ -271,16 +257,16 @@ long WorkspacesPane::limitWidth(long paneWidth) {
             }
             if (*end == '%' && inrange(num, 0L, 100L)) {
                 maxPercent = num;
-                maxPixels = maxPercent * desktop->width() / 100L;
+                maxPixels = maxPercent * taskBarWidth / 100L;
                 maxButtons = maxPixels * count() / non_zero(paneWidth);
             }
         }
     }
     if (isEmpty(str) || 0 == (maxPixels | maxButtons)) {
         if (taskBarDoubleHeight && taskBarWorkspacesTop) {
-            maxPixels = long(desktop->width()) - x() - 20L;
+            maxPixels = taskBarWidth - x() - reserved;
         } else {
-            maxPixels = (long(desktop->width()) - x() - 20L) / 2;
+            maxPixels = (taskBarWidth - x() - reserved) / 2;
             maxButtons = 20;    // old limit
         }
     }
@@ -351,6 +337,7 @@ void WorkspacesPane::relabelButtons() {
         repositionButtons();
 
     paths = null;
+    repaint();
 }
 
 void WorkspacesPane::configure(const YRect2& r) {
@@ -400,8 +387,10 @@ WorkspaceButton* WorkspacesPane::create(int workspace, unsigned height) {
     WorkspaceButton *wk = new WorkspaceButton(workspace, this, this);
     fButtons += wk;
     if (pagerShowPreview) {
-        double scaled = double(height * desktop->width()) / desktop->height();
-        wk->setSize(unsigned(lround(scaled)), height);
+        unsigned dw = desktop->width();
+        unsigned dh = desktop->height();
+        unsigned scaled = (height * dw + (dh / 2)) / dh;
+        wk->setSize(scaled, height);
         wk->updateName();
     } else {
         label(wk);
@@ -435,7 +424,7 @@ ref<YImage> WorkspaceIcons::load(const char* name) {
             }
         }
     }
-    cstring trim(mstring(name).trim());
+    mstring trim(mstring(name).trim());
     return (trim.length() && trim != name) ? load(trim) : null;
 }
 
@@ -539,18 +528,31 @@ void WorkspacesPane::stopDrag() {
     }
 }
 
+ref<YFont> WorkspaceButton::getActiveFont() {
+    if (activeButtonFont == null) {
+        if (*activeWorkspaceFontName || *activeWorkspaceFontNameXft) {
+            activeButtonFont = YFont::getFont(XFA(activeWorkspaceFontName));
+        }
+        if (activeButtonFont == null) {
+            activeButtonFont = YButton::getActiveFont();
+        }
+    }
+    return activeButtonFont;
+}
+
 ref<YFont> WorkspaceButton::getFont() {
-    return isPressed()
-        ? (*activeWorkspaceFontName || *activeWorkspaceFontNameXft)
-        ? activeButtonFont != null
-        ? activeButtonFont
-        : activeButtonFont = YFont::getFont(XFA(activeWorkspaceFontName))
-        : YButton::getFont()
-        : (*normalWorkspaceFontName || *normalWorkspaceFontNameXft)
-        ? normalButtonFont != null
-        ? normalButtonFont
-        : normalButtonFont = YFont::getFont(XFA(normalWorkspaceFontName))
-        : YButton::getFont();
+    if (isPressed()) {
+        return getActiveFont();
+    }
+    else if (normalButtonFont == null) {
+        if (*normalWorkspaceFontName || *normalWorkspaceFontNameXft) {
+            normalButtonFont = YFont::getFont(XFA(normalWorkspaceFontName));
+        }
+        if (normalButtonFont == null) {
+            normalButtonFont = YButton::getFont();
+        }
+    }
+    return normalButtonFont;
 }
 
 YColor WorkspaceButton::getColor() {
@@ -569,6 +571,11 @@ YSurface WorkspaceButton::getSurface() {
                        workspacebuttonPixbuf));
 }
 
+YDimension WorkspaceButton::getTextSize() {
+    ref<YFont> font(getActiveFont());
+    return YDimension(font->textWidth(name()), font->height());
+}
+
 const char* WorkspaceButton::name() const {
     return workspaces[fWorkspace];
 }
@@ -594,8 +601,6 @@ void WorkspacesPane::repaint() {
 }
 
 void WorkspaceButton::paint(Graphics &g, const YRect& r) {
-    paintBackground(g, r);
-
     if (!pagerShowPreview) {
         YButton::paint(g, r);
         return;
@@ -612,9 +617,6 @@ void WorkspaceButton::paint(Graphics &g, const YRect& r) {
             x += 1; y += 1; w -= 2; h -= 2;
         }
 
-        unsigned wx, wy, ww, wh;
-        double sf = (double) desktop->width() / w;
-
         ref<YIcon> icon;
         YColor colors[] = {
             surface.color,
@@ -629,16 +631,22 @@ void WorkspaceButton::paint(Graphics &g, const YRect& r) {
                 yfw && yfw->getActiveLayer() <= WinLayerDock;
                 yfw = yfw->prevLayer()) {
             if (yfw->isHidden() ||
-                    !yfw->visibleOn(fWorkspace) ||
-                    hasbit(yfw->frameOptions(),
-                        YFrameWindow::foIgnoreWinList |
-                        YFrameWindow::foIgnorePagerPreview))
+                hasbit(yfw->frameOptions(),
+                       YFrameWindow::foIgnoreWinList |
+                       YFrameWindow::foIgnorePagerPreview)) {
                 continue;
-            wx = (unsigned) round(double(yfw->x()) / sf) + x;
-            wy = (unsigned) round(double(yfw->y()) / sf) + y;
-            ww = (unsigned) round(double(yfw->width()) / sf);
-            wh = (unsigned) round(double(yfw->height()) / sf);
-            if (ww < 1 || wh < 1)
+            }
+            if (yfw->isAllWorkspaces() ?
+                fWorkspace != manager->activeWorkspace() :
+                fWorkspace != yfw->getWorkspace()) {
+                continue;
+            }
+            int dw = int(desktop->width());
+            int wx = x + (yfw->x() * w + (dw / 2)) / dw;
+            int wy = y + (yfw->y() * w + (dw / 2)) / dw;
+            int ww = (int(yfw->width()) * w + (dw / 2)) / dw;
+            int wh = (int(yfw->height()) * w + (dw / 2)) / dw;
+            if (ww <= 1 || wh <= 1)
                 continue;
             if (yfw->isMaximizedVert()) { // !!! hack
                 wy = y; wh = h;
@@ -655,9 +663,12 @@ void WorkspaceButton::paint(Graphics &g, const YRect& r) {
                         g.setColor(colors[2]);
                     g.fillRect(wx+1, wy+1, ww-2, wh-2);
 
-                    if (pagerShowWindowIcons && ww > smallIconSize+1 &&
-                            wh > smallIconSize+1 && (icon = yfw->clientIcon()) != null &&
-                            icon->small() != null) {
+                    if (pagerShowWindowIcons &&
+                        ww > 1 + int(smallIconSize) &&
+                        wh > 1 + int(smallIconSize) &&
+                        (icon = yfw->clientIcon()) != null &&
+                        icon->small() != null)
+                    {
                         g.drawImage(icon->small(),
                                     wx + (ww-smallIconSize)/2,
                                     wy + (wh-smallIconSize)/2);
@@ -679,14 +690,15 @@ void WorkspaceButton::paint(Graphics &g, const YRect& r) {
         char label[12] = {};
         if (pagerShowNumbers) {
             snprintf(label, sizeof label, "%d", int(fWorkspace+1) % 100);
-        } else {
-            strlcpy(label, cstring(baseName()), min(5, int(sizeof label)));
+        }
+        else if (pagerShowLabels) {
+            strlcpy(label, baseName(), min(5, int(sizeof label)));
         }
         if (label[0] != 0) {
             ref<YFont> font = getFont();
 
-            wx = (w - font->textWidth(label)) / 2 + x;
-            wy = (h - font->height()) / 2 + font->ascent() + y;
+            int wx = (w - font->textWidth(label)) / 2 + x;
+            int wy = (h - font->height()) / 2 + font->ascent() + y;
 
             g.setFont(font);
             g.setColor(colors[0]);
@@ -694,6 +706,9 @@ void WorkspaceButton::paint(Graphics &g, const YRect& r) {
             g.setColor(colors[3]);
             g.drawChars(label, 0, strlen(label), wx, wy);
         }
+        if (surface.gradient != null) {
+            g.maxOpacity();
+        }
     }
 }
 
