@@ -1,31 +1,40 @@
 #ifndef __OBJ_H
 #define __OBJ_H
 
-#include "ypaint.h"
+#include "mstring.h"
+
+class IApp;
+class YIcon;
 
 class DObject {
 public:
-    DObject(const ustring &name, ref<YIcon> icon);
+    DObject(IApp *app, const mstring &name, ref<YIcon> icon);
     virtual ~DObject();
 
-    ustring getName() { return fName; }
+    mstring getName() { return fName; }
     ref<YIcon> getIcon() { return fIcon; }
 
     virtual void open();
 
 private:
-    ustring fName;
+    mstring fName;
     ref<YIcon> fIcon;
+protected:
+    IApp *app;
 };
 
+class ObjectMenu;
+
 class ObjectContainer {
 public:
     virtual void addObject(DObject *object) = 0;
     virtual void addSeparator() = 0;
-    virtual void addContainer(const ustring &name, ref<YIcon> icon, ObjectContainer *container) = 0;
+    virtual void addContainer(const mstring &name, ref<YIcon> icon, ObjectMenu *container) = 0;
 protected:
-    virtual ~ObjectContainer() {};
+    virtual ~ObjectContainer() {}
 };
 
 
 #endif
+
+// vim: set sw=4 ts=4 et:
