@@ -12,19 +12,19 @@
 #include "default.h"
 #include "yxapp.h"
 #include "wmprog.h"
-
-#ifndef LITE
+#include "ypaths.h"
+#include "ypointer.h"
 
 #include <limits.h>
 #include <stdlib.h>
 
 #ifdef CONFIG_XPM
-#include "X11/xpm.h"
-#endif
-
-#ifdef CONFIG_IMLIB
+#include <X11/xpm.h>
+#elif defined CONFIG_IMLIB
 #include <Imlib.h>
 extern ImlibData *hImlib;
+#else
+#error "Need either XPM or Imlib for cursors."
 #endif
 
 #include "ycursor.h"
@@ -41,51 +41,59 @@ public:
     const XColor& foreground() const { return fForeground; }
 
 #ifdef CONFIG_XPM
+
     bool isValid() { return fValid; }
     unsigned int width() const { return fAttributes.width; }
     unsigned int height() const { return fAttributes.height; }
     unsigned int hotspotX() const { return fAttributes.x_hotspot; }
     unsigned int hotspotY() const { return fAttributes.y_hotspot; }
-#endif
 
-#ifdef CONFIG_IMLIB
+#elif defined CONFIG_IMLIB
+
     bool isValid() { return fImage; }
     unsigned int width() const { return fImage ? fImage->rgb_width : 0; }
     unsigned int height() const { return fImage ? fImage->rgb_height : 0; }
     unsigned int hotspotX() const { return fHotspotX; }
     unsigned int hotspotY() const { return fHotspotY; }
-#endif
 
-#ifdef CONFIG_GDK_PIXBUF_XLIB
+#elif defined CONFIG_GDK_PIXBUF_XLIB
+
     bool isValid() { return false; }
     unsigned int width() const { return 0; }
     unsigned int height() const { return 0; }
     unsigned int hotspotX() const { return fHotspotX; }
     unsigned int hotspotY() const { return fHotspotY; }
+
 #endif
 
+    operator bool();
+
 private:
     Pixmap fPixmap, fMask;
     XColor fForeground, fBackground;
 
 #ifdef CONFIG_XPM
+
     bool fValid;
     XpmAttributes fAttributes;
-#endif
 
-#ifdef CONFIG_IMLIB
+#elif defined CONFIG_IMLIB
+
     unsigned int fHotspotX, fHotspotY;
     ImlibImage *fImage;
-#endif
 
-#ifdef CONFIG_GDK_PIXBUF_XLIB
+#elif defined CONFIG_GDK_PIXBUF_XLIB
+
     bool fValid;
     unsigned int fHotspotX, fHotspotY;
+
 #endif
-    operator bool();
 };
 
-#ifdef CONFIG_XPM // ================== use libXpm to load the cursor pixmap ===
+#ifdef CONFIG_XPM
+//
+// ================== use libXpm to load the cursor pixmap ===
+//
 YCursorPixmap::YCursorPixmap(upath path): fValid(false) {
     fAttributes.colormap  = xapp->colormap();
     fAttributes.closeness = 65535;
@@ -93,17 +101,21 @@ YCursorPixmap::YCursorPixmap(upath path): fValid(false) {
                             XpmReturnPixels|XpmSize|XpmHotspot;
     fAttributes.x_hotspot = 0;
     fAttributes.y_hotspot = 0;
+    fAttributes.depth = 0;
+    fAttributes.width = 0;
+    fAttributes.height = 0;
 
+    csmart filename(newstr(path.string()));
     int const rc(XpmReadFileToPixmap(xapp->display(), desktop->handle(),
-                                     (char *)REDIR_ROOT(cstring(path.path()).c_str()), // !!!
+                                     filename, // !!!
                                      &fPixmap, &fMask, &fAttributes));
 
     if (rc != XpmSuccess)
         warn(_("Loading of pixmap \"%s\" failed: %s"),
-               path, XpmGetErrorString(rc));
+               path.string(), XpmGetErrorString(rc));
     else if (fAttributes.npixels != 2)
         warn("Invalid cursor pixmap: \"%s\" contains too many unique colors",
-               path);
+               path.string());
     else {
         fBackground.pixel = fAttributes.pixels[0];
         fForeground.pixel = fAttributes.pixels[1];
@@ -114,24 +126,26 @@ YCursorPixmap::YCursorPixmap(upath path): fValid(false) {
         fValid = true;
     }
 }
-#endif
 
-#ifdef CONFIG_IMLIB // ================= use Imlib to load the cursor pixmap ===
+#elif defined CONFIG_IMLIB
+//
+// ================= use Imlib to load the cursor pixmap ===
+//
 YCursorPixmap::YCursorPixmap(upath path):
     fHotspotX(0), fHotspotY(0)
 {
-    cstring cs(path.path());
-    fImage = Imlib_load_image(hImlib, (char *)REDIR_ROOT(cs.c_str()));
+    mstring cs(path.path());
+    fImage = Imlib_load_image(hImlib, (char *)cs.c_str());
 
     if (fImage == NULL) {
         warn(_("Loading of pixmap \"%s\" failed"), cs.c_str());
         return;
     }
-    
+
     Imlib_render(hImlib, fImage, fImage->rgb_width, fImage->rgb_height);
     fPixmap = (Pixmap)Imlib_move_image(hImlib, fImage);
     fMask = (Pixmap)Imlib_move_mask(hImlib, fImage);
-        
+
     struct Pixel { // ----------------- find the background/foreground color ---
         bool operator!= (const Pixel& o) {
             return (r != o.r || g != o.g || b != o.b); }
@@ -165,7 +179,7 @@ YCursorPixmap::YCursorPixmap(upath path):
                         return;
                     }
             }
-    
+
     fForeground.red = (unsigned short)(fg.r << 8); // -- alloc the background/foreground color ---
     fForeground.green = (unsigned short)(fg.g << 8);
     fForeground.blue = (unsigned short)(fg.b << 8);
@@ -177,7 +191,7 @@ YCursorPixmap::YCursorPixmap(upath path):
     XAllocColor(xapp->display(), xapp->colormap(), &fBackground);
 
     // ----------------- find the hotspot by reading the xpm header manually ---
-    FILE *xpm = fopen((char *)REDIR_ROOT(cs.c_str()), "rb");
+    FILE *xpm = path.fopen("rb");
     if (xpm == NULL)
         warn(_("BUG? Imlib was able to read \"%s\""), cs.c_str());
 
@@ -189,7 +203,7 @@ YCursorPixmap::YCursorPixmap(upath path):
                 if ((c == fgetc(xpm)) == '/') // ------ eat C++ line comment ---
                     while (fgetc(xpm) != '\n');
                 else { // -------------------------------- eat block comment ---
-                   int pc; do { pc = c; c = fgetc(xpm); } 
+                   int pc; do { pc = c; c = fgetc(xpm); }
                    while (c != '/' && pc != '*');
                 }
                 break;
@@ -216,7 +230,7 @@ YCursorPixmap::YCursorPixmap(upath path):
                     warn(_("BUG? Unexpected end of XPM file but Imlib "
                            "was able to parse \"%s\""), cs.c_str());
                 else
-                    warn(_("BUG? Unexpected characted but Imlib "
+                    warn(_("BUG? Unexpected character but Imlib "
                            "was able to parse \"%s\""), cs.c_str());
 
                 fclose(xpm);
@@ -224,9 +238,9 @@ YCursorPixmap::YCursorPixmap(upath path):
         }
     }
 }
-#endif
 
-#ifdef CONFIG_GDK_PIXBUF_XLIB
+#elif defined CONFIG_GDK_PIXBUF_XLIB
+
 YCursorPixmap::YCursorPixmap(upath /*path*/):
     fPixmap(None), fMask(None),
     fHotspotX(0), fHotspotY(0)
@@ -240,36 +254,48 @@ YCursorPixmap::~YCursorPixmap() {
     if (fMask != None)
         XFreePixmap(xapp->display(), fMask);
 
-#ifdef CONFIG_XPM 
+#ifdef CONFIG_XPM
+
     XpmFreeAttributes(&fAttributes);
-#endif
+
+#elif defined CONFIG_IMLIB
 
-#ifdef CONFIG_IMLIB
     Imlib_destroy_image(hImlib, fImage);
+
 #endif
 }
 
-#endif
-
 YCursor::~YCursor() {
-    if(fOwned && fCursor && app)
-        XFreeCursor(xapp->display(), fCursor);
+    unload();
 }
 
-#ifndef LITE
+class MyCursorLoader : public YCursorLoader {
+private:
+    ref<YResourcePaths> paths;
+
+    Cursor load(upath path);
+
+public:
+    MyCursorLoader()
+        : paths(YResourcePaths::subdirs("cursors/"))
+    { }
+
+    virtual Cursor load(upath path, unsigned int fallback);
+};
 
 static Pixmap createMask(int w, int h) {
     return XCreatePixmap(xapp->display(), desktop->handle(), w, h, 1);
 }
 
-void YCursor::load(upath path) {
+Cursor MyCursorLoader::load(upath path) {
+    Cursor fCursor = None;
     YCursorPixmap pixmap(path);
-    
+
     if (pixmap.isValid()) { // ============ convert coloured pixmap into a bilevel one ===
         Pixmap bilevel(createMask(pixmap.width(), pixmap.height()));
 
         // -------------------------- figure out which plane we have to copy ---
-        unsigned long pmask(1 << (xapp->depth() - 1));
+        unsigned long pmask(1UL << (xapp->depth() - 1));
 
         if (pixmap.foreground().pixel &&
             pixmap.foreground().pixel != pixmap.background().pixel)
@@ -279,7 +305,7 @@ void YCursor::load(upath path) {
             while ((pixmap.background().pixel & pmask) == 0) pmask >>= 1;
 
         GC gc; XGCValues gcv; // ------ copy one plane by using a bilevel GC ---
-        gcv.function = (pixmap.foreground().pixel && 
+        gcv.function = (pixmap.foreground().pixel &&
                        (pixmap.foreground().pixel & pmask))
                      ? GXcopyInverted : GXcopy;
         gc = XCreateGC (xapp->display(), bilevel, GCFunction, &gcv);
@@ -296,36 +322,51 @@ void YCursor::load(upath path) {
                background(pixmap.background());
 
         fCursor = XCreatePixmapCursor(xapp->display(),
-                                      bilevel, pixmap.mask(), 
+                                      bilevel, pixmap.mask(),
                                       &foreground, &background,
                                       pixmap.hotspotX(), pixmap.hotspotY());
 
         XFreePixmap(xapp->display(), bilevel);
     }
+    return fCursor;
 }
-#endif
 
-#ifndef LITE
-void YCursor::load(upath name, unsigned int fallback) {
-#else
-void YCursor::load(upath /*name*/, unsigned int fallback) {
-#endif
-    if(fCursor && fOwned)
-        XFreeCursor(xapp->display(), fCursor);
+void YCursor::unload() {
+    if (fOwned) {
+        fOwned = false;
+        if (fCursor) {
+            if (xapp) {
+                XFreeCursor(xapp->display(), fCursor);
+            }
+            fCursor = None;
+        }
+    }
+}
 
-#ifndef LITE
-    char const *cursors = "cursors/";
-    ref<YResourcePaths> paths = YResourcePaths::subdirs(cursors);
+YCursorLoader* YCursor::newLoader() {
+    return new MyCursorLoader();
+}
+
+Cursor MyCursorLoader::load(upath name, unsigned int fallback)
+{
+    Cursor fCursor = None;
+
+    upath cursors("cursors/");
 
     for (int i = 0; i < paths->getCount(); i++) {
-        upath path = paths->getPath(i)->joinPath(cursors, name);
-        if (path.fileExists())
-            load(path.path());
+        upath path = paths->getPath(i) + cursors + name;
+        if (path.fileExists()) {
+            if ((fCursor = load(path)) != None) {
+                /* stop when successful */
+                break;
+            }
+        }
     }
 
     if (fCursor == None)
-#endif    
         fCursor = XCreateFontCursor(xapp->display(), fallback);
-        
-    fOwned = true;
+
+    return fCursor;
 }
+
+// vim: set sw=4 ts=4 et:
