@@ -3,6 +3,9 @@
 
 #include "ylib.h"
 
+#include <X11/XKBlib.h>
 #include <X11/keysym.h>
 
 #endif
+
+// vim: set sw=4 ts=4 et:
