@@ -26,180 +26,132 @@ static YColorName listBoxFg(&clrListBoxText);
 static YColorName listBoxSelBg(&clrListBoxSelected);
 static YColorName listBoxSelFg(&clrListBoxSelectedText);
 
+inline int getIconSize() {
+    return int(YIcon::smallSize());
+}
+
+inline ref<YFont> getListBoxFont() {
+    if (listBoxFont == null)
+        listBoxFont = YFont::getFont(XFA(listBoxFontName));
+    return listBoxFont;
+}
+
 int YListBox::fAutoScrollDelta = 0;
 
-YListItem::YListItem() {
-    fPrevItem = fNextItem = 0;
-    fSelected = false;
-}
-
-YListItem::~YListItem() {
-}
-
-YListItem *YListItem::getNext() {
-    return fNextItem;
-}
-
-YListItem *YListItem::getPrev() {
-    return fPrevItem;
-}
-
-void YListItem::setNext(YListItem *next) {
-    fNextItem = next;
-}
-
-void YListItem::setPrev(YListItem *prev) {
-    fPrevItem = prev;
-}
-
-void YListItem::setSelected(bool aSelected) {
-    fSelected = aSelected;
-}
-
-ustring YListItem::getText() {
-    return null;
-}
-
-ref<YIcon> YListItem::getIcon() {
-    return null;
-}
-
-int YListItem::getOffset() {
-    return 0;
+int YListItem::getWidth() {
+    int width = 3 + 20 + getOffset();
+    if (getIcon() != null) {
+        width += getIconSize();
+    }
+    if (getText() != null) {
+        width += getListBoxFont()->textWidth(getText()) + 3;
+    }
+    return width;
 }
 
 YListBox::YListBox(YScrollView *view, YWindow *aParent):
     YWindow(aParent),
+    fVerticalScroll(nullptr),
+    fHorizontalScroll(nullptr),
+    fView(view),
+    fOffsetX(0),
+    fOffsetY(0),
+    fMaxWidth(0),
+    fWidestItem(-1),
+    fFocusedItem(0),
+    fSelectStart(-1),
+    fSelectEnd(-1),
+    fDragging(false),
+    fSelect(false),
+    fVisible(false),
+    fOutdated(false),
     fGraphics(this),
     fGradient(null)
 {
     if (listBoxFont == null)
         listBoxFont = YFont::getFont(XFA(listBoxFontName));
-    setStyle(wsNoExpose);
+    addStyle(wsNoExpose);
     setBitGravity(NorthWestGravity);
     addEventMask(VisibilityChangeMask);
-    fView = view;
     if (fView) {
         fVerticalScroll = view->getVerticalScrollBar();
         fHorizontalScroll = view->getHorizontalScrollBar();
-    } else {
-        fHorizontalScroll = 0;
-        fVerticalScroll = 0;
     }
     if (fVerticalScroll)
         fVerticalScroll->setScrollBarListener(this);
     if (fHorizontalScroll)
         fHorizontalScroll->setScrollBarListener(this);
-    fOffsetX = 0;
-    fOffsetY = 0;
-    fFirst = fLast = 0;
-    fFocusedItem = 0;
-    fSelectStart = fSelectEnd = -1;
-    fDragging = false;
-    fSelect = false;
-    fVisible = false;
-    fOutdated = false;
-    fItemCount = 0;
-    fItems = 0;
     setTitle("ListBox");
 }
 
 YListBox::~YListBox() {
-    fFirst = fLast = 0;
-    freeItems();
     fGradient = null;
+    listBoxFont = null;
 }
 
 bool YListBox::isFocusTraversable() {
     return true;
 }
 
-void YListBox::addAfter(YListItem *prev, YListItem *item) {
-    PRECONDITION(item->getPrev() == 0);
-    PRECONDITION(item->getNext() == 0);
+void YListBox::addAfter(YListItem *after, YListItem *item) {
+    int i = findItem(after);
+    if (i >= 0) {
+        fItems.insert(i + 1, item);
+        if (fFocusedItem > i)
+            fFocusedItem++;
+        if (fWidestItem > i)
+            fWidestItem++;
+    } else {
+        fItems.append(item);
+    }
+    outdated();
+}
 
-    freeItems();
-
-    item->setNext(prev->getNext());
-    if (item->getNext())
-        item->getNext()->setPrev(item);
-    else
-        fLast = item;
-
-    item->setPrev(prev);
-    prev->setNext(item);
-    fItemCount++;
+void YListBox::addBefore(YListItem *before, YListItem *item) {
+    int i = findItem(before);
+    if (i >= 0) {
+        fItems.insert(i, item);
+        if (fFocusedItem >= i)
+            fFocusedItem++;
+        if (fWidestItem >= i)
+            fWidestItem++;
+    } else {
+        fItems.append(item);
+    }
     outdated();
 }
 
 void YListBox::addItem(YListItem *item) {
-    PRECONDITION(item->getPrev() == 0);
-    PRECONDITION(item->getNext() == 0);
-
-    freeItems();
-
-    item->setNext(0);
-    item->setPrev(fLast);
-    if (fLast)
-        fLast->setNext(item);
-    else
-        fFirst = item;
-    fLast = item;
-    fItemCount++;
+    fItems.append(item);
     outdated();
 }
 
 void YListBox::removeItem(YListItem *item) {
-    freeItems();
-
-    bool focused = (inrange(fFocusedItem, 0, getItemCount() - 1)
-                    && fItems && fItems[fFocusedItem] == item);
-    if (item->getPrev())
-        item->getPrev()->setNext(item->getNext());
-    else
-        fFirst = item->getNext();
-
-    if (item->getNext())
-        item->getNext()->setPrev(item->getPrev());
-    else
-        fLast = item->getPrev();
-    item->setPrev(0);
-    item->setNext(0);
-    fItemCount--;
-
-    if (focused || fFocusedItem >= fItemCount)
-        fFocusedItem = -1;
-    outdated();
-}
-
-void YListBox::freeItems() {
-    if (fItems) {
-        delete[] fItems; fItems = 0;
+    int index = findItem(item);
+    if (index >= 0) {
+        fItems.remove(index);
+        if (fFocusedItem > index)
+            fFocusedItem--;
+        else if (index == fFocusedItem) {
+            while (index-- && 0 < fItems[index]->getOffset()) { }
+            fFocusedItem = index;
+        }
+        outdated();
     }
 }
 
 void YListBox::updateItems() {
-    if (fItems == 0) {
+    if (fOutdated) {
         fMaxWidth = 0;
-        fItems = new YListItem *[fItemCount];
-        if (fItems) {
-            YListItem *a = getFirst();
-            int n = 0;
-            while (a) {
-                fItems[n++] = a;
-
-                int cw = 3 + 20 + a->getOffset();
-                if (listBoxFont != null) {
-                    ustring t = a->getText();
-                    if (t != null)
-                        cw += listBoxFont->textWidth(t) + 3;
-                }
-                if (cw > fMaxWidth)
-                    fMaxWidth = cw;
-
-                a = a->getNext();
+        fWidestItem = -1;
+        for (IterType a(getIterator()); ++a; ) {
+            int width = a->getWidth();
+            if (width > fMaxWidth) {
+                fMaxWidth = width;
+                fWidestItem = a.where();
             }
         }
+        fOutdated = false;
     }
 }
 
@@ -210,44 +162,15 @@ int YListBox::maxWidth() {
 
 int YListBox::findItemByPoint(int /*pX*/, int pY) {
     int no = (pY + fOffsetY) / getLineHeight();
-
-    if (no >= 0 && no < getItemCount())
-        return no;
-    return -1;
+    return (0 <= no && no < getItemCount()) ? no : -1;
 }
 
-int YListBox::getItemCount() {
-    return fItemCount;
-}
-
-int YListBox::findItem(YListItem *item) {
-    YListItem *a = fFirst;
-    int n;
-
-    for (n = 0; a; a = a->getNext(), n++)
-        if (item == a)
-            return n;
-    return -1;
- }
-
 YListItem *YListBox::getItem(int no) {
-    if (no < 0 || no >= getItemCount())
-        return 0;
-    updateItems();
-
-    if (fItems) {
-        return fItems[no];
-    } else {
-        YListItem *a = getFirst();
-        for (int n = 0; a; a = a->getNext(), n++)
-            if (n == no)
-                return a;
-    }
-    return 0;
+    return (0 <= no && no < getItemCount()) ? fItems[no] : nullptr;
 }
 
 int YListBox::getLineHeight() {
-    return max((int) YIcon::smallSize(), (int) listBoxFont->height()) + 2;
+    return max(getIconSize(), listBoxFont->height()) + 2;
 }
 
 void YListBox::ensureVisibility(int item) { //!!! horiz too
@@ -282,12 +205,10 @@ void YListBox::focusVisible() {
 }
 
 void YListBox::configure(const YRect2& r) {
-    resetScrollBars();
-
     if (listbackPixbuf != null
-        && !(fGradient != null &&
-             fGradient->width() == r.width() &&
-             fGradient->height() == r.height()))
+        && (fGradient == null ||
+            fGradient->width() != r.width() ||
+            fGradient->height() != r.height()))
     {
         fGradient = listbackPixbuf->scale(r.width(), r.height());
     }
@@ -309,9 +230,7 @@ void YListBox::handleVisibility(const XVisibilityEvent& visib) {
 
 bool YListBox::handleTimer(YTimer* timer) {
     if (timer == fTimer) {
-        if (fOutdated && fVisible) {
-            repaint();
-        }
+        repaint();
     }
     return false;
 }
@@ -323,8 +242,9 @@ void YListBox::outdated() {
 
 void YListBox::repaint() {
     if (fVisible) {
+        updateItems();
+        resetScrollBars();
         fGraphics.paint();
-        fOutdated = false;
     }
 }
 
@@ -333,8 +253,8 @@ bool YListBox::handleKey(const XKeyEvent &key) {
         KeySym k = keyCodeToKeySym(key.keycode);
         int m = KEY_MODMASK(key.state);
 
-        bool clear = (m & ControlMask) ? false : true;
-        bool extend = (m & ShiftMask) ? true : false;
+        bool clear = notbit(m, ControlMask);
+        bool extend = hasbit(m, ShiftMask);
 
         //int SelPos, OldPos = fFocusedItem, count = getItemCount();
 
@@ -356,7 +276,7 @@ bool YListBox::handleKey(const XKeyEvent &key) {
         case ' ':
             if (fFocusedItem != -1) {
                 selectItem(fFocusedItem,
-                           isItemSelected(fFocusedItem) ? false : true);
+                           isSelected(fFocusedItem) ? false : true);
             }
             break;
         case XK_Home:
@@ -415,22 +335,27 @@ bool YListBox::handleKey(const XKeyEvent &key) {
         case '\\':
             if (m & ControlMask) {
                 for (int i = 0; i < getItemCount(); i++)
-                    selectItem(i, (k == '\\') ? false : true);
+                    selectItem(i, (k != '\\'));
                 break;
             }
-            break;
+            /* fall-through */
         default:
             if (k < 256) {
-                unsigned char c = ASCII::toUpper((char)k);
-                int count = getItemCount();
-                int i = fFocusedItem;
-                YListItem *it = 0;
+                const int ksize = 16;
+                char kstr[ksize] = { char(k), 0, };
+                getCharFromEvent(key, kstr, ksize);
+                const int klen = int(strnlen(kstr, ksize));
+                if (ASCII::isControl(kstr[0]) && kstr[0] != char(k)) {
+                    extend = false;
+                }
 
-                for (int n = 0; n < count; n++) {
-                    i = (i + 1) % count;
-                    it = getItem(i);
-                    ustring title = it->getText();
-                    if (title != null && title.length() > 0 && ASCII::toUpper(title.charAt(0)) == c) {
+                for (int n = 0; n < 2 * getItemCount(); ++n) {
+                    int i = (fFocusedItem + 1 + n) % getItemCount();
+                    mstring title(getItem(i)->getText());
+                    if (n < getItemCount()
+                        ? strncasecmp(title, kstr, klen) == 0
+                        : strstr(title, kstr) != nullptr)
+                    {
                         setFocusedItem(i, clear, extend, false);
                         break;
                     }
@@ -479,11 +404,11 @@ void YListBox::handleButton(const XButtonEvent &button) {
         int no = findItemByPoint(button.x, button.y);
 
         if (button.type == ButtonPress) {
-            bool clear = (button.state & ControlMask) ? false : true;
-            bool extend = (button.state & ShiftMask) ? true : false;
+            bool clear = notbit(button.state, ControlMask);
+            bool extend = hasbit(button.state, ShiftMask);
 
             if (no != -1) {
-                fSelect = (!clear && isItemSelected(no)) ? false : true;
+                fSelect = (!clear && isSelected(no)) ? false : true;
                 setFocusedItem(no, clear, extend, true);
             } else {
                 fSelect = true;
@@ -494,7 +419,7 @@ void YListBox::handleButton(const XButtonEvent &button) {
                 setFocusedItem(no, false, true, true);
             fDragging = false;
             applySelection();
-            autoScroll(0, 0);
+            autoScroll(0, nullptr);
         }
     }
     if (fVerticalScroll->handleScrollMouse(button) == false)
@@ -575,12 +500,12 @@ void YListBox::paintItem(Graphics &g, int n) {
     YListItem *a = getItem(n);
     int x = 3;
     int lh = getLineHeight();
-    int fh = listBoxFont->height();
+    int fh = getListBoxFont()->height();
     int xpos = 0;
     int y = n * lh;
     int yPos = y + lh - (lh - fh) / 2 - listBoxFont->descent();
 
-    if (a == 0)
+    if (a == nullptr)
         return ;
 
     xpos += a->getOffset();
@@ -588,8 +513,8 @@ void YListBox::paintItem(Graphics &g, int n) {
     bool s(a->getSelected());
 
     if (fDragging) {
-        int const beg(fSelectStart < fSelectEnd ? fSelectStart : fSelectEnd);
-        int const end(fSelectStart < fSelectEnd ? fSelectEnd : fSelectStart);
+        int const beg(min(fSelectStart, fSelectEnd));
+        int const end(max(fSelectStart, fSelectEnd));
 
         if (n >= beg && n <= end)
             s = fSelect;
@@ -616,7 +541,7 @@ void YListBox::paintItem(Graphics &g, int n) {
         g.setPenStyle(true);
         int cw = 3 + 20 + a->getOffset();
         if (listBoxFont != null) {
-            ustring t = a->getText();
+            mstring t = a->getText();
             if (t != null)
                 cw += listBoxFont->textWidth(t) + 3;
         }
@@ -626,7 +551,7 @@ void YListBox::paintItem(Graphics &g, int n) {
 
     ref<YIcon> icon = a->getIcon();
     if (icon != null) {
-        ref<YImage> scaled = icon->getScaledIcon(YIcon::menuSize());
+        ref<YImage> scaled = icon->getScaledIcon(getIconSize());
         if (scaled != null) {
             int dx = xpos + x - fOffsetX;
             int dy = y - fOffsetY + 1;
@@ -634,7 +559,7 @@ void YListBox::paintItem(Graphics &g, int n) {
         }
     }
 
-    ustring title = a->getText();
+    mstring title = a->getText();
 
     if (title != null) {
         g.setColor(s ? listBoxSelFg : listBoxFg);
@@ -651,8 +576,9 @@ void YListBox::paint(Graphics &g, const YRect &r) {
     int const min((fOffsetY + ry) / lh);
     int const max((fOffsetY + ry + rheight) / lh);
 
-    for (int n(min); n <= max; n++) paintItem(g, n);
-    resetScrollBars();
+    for (int n(min); n <= max; n++) {
+        paintItem(g, n);
+    }
 
     unsigned const y(contentHeight());
 
@@ -684,8 +610,17 @@ void YListBox::activateItem(YListItem *item) {
 
 void YListBox::repaintItem(YListItem *item) {
     int i = findItem(item);
-    if (i != -1)
+    if (i != -1) {
+        if (i == fWidestItem) {
+            if (item->getWidth() != fMaxWidth) {
+                outdated();
+            }
+        }
+        else if (item->getWidth() > fMaxWidth) {
+            outdated();
+        }
         paintItem(i);
+    }
 }
 
 bool YListBox::hasSelection() {//!!!fix
@@ -728,13 +663,6 @@ void YListBox::autoScroll(int delta, const XMotionEvent *motion) {
     beginAutoScroll(delta ? true : false, motion);
 }
 
-bool YListBox::isItemSelected(int item) {
-    YListItem *i = getItem(item);
-    if (i)
-        return i->getSelected();
-    return false;
-}
-
 void YListBox::selectItem(int item, bool select) {
     YListItem *i = getItem(item);
     if (i && i->getSelected() != select) {
@@ -755,8 +683,8 @@ void YListBox::applySelection() {
     if (fSelectStart != -1) {
         PRECONDITION(fSelectEnd != -1);
 
-        int beg = (fSelectStart < fSelectEnd) ? fSelectStart : fSelectEnd;
-        int end = (fSelectStart < fSelectEnd) ? fSelectEnd : fSelectStart;
+        int beg = min(fSelectStart, fSelectEnd);
+        int end = max(fSelectStart, fSelectEnd);
 
         for (int n = beg; n <= end ; n++) {
             YListItem *i = getItem(n);
@@ -774,8 +702,8 @@ void YListBox::paintItems(int selStart, int selEnd) {
     //PRECONDITION(selStart != -1);
     PRECONDITION(selEnd != -1);
 
-    int beg = (selStart < selEnd) ? selStart : selEnd;
-    int end = (selStart < selEnd) ? selEnd : selStart;
+    int beg = min(selStart, selEnd);
+    int end = max(selStart, selEnd);
 
     for (int i = beg; i <= end; i++)
         paintItem(i);
@@ -785,8 +713,8 @@ void YListBox::selectItems(int selStart, int selEnd, bool sel) {
     PRECONDITION(selStart != -1);
     PRECONDITION(selEnd != -1);
 
-    int beg = (selStart < selEnd) ? selStart : selEnd;
-    int end = (selStart < selEnd) ? selEnd : selStart;
+    int beg = min(selStart, selEnd);
+    int end = max(selStart, selEnd);
 
     for (int i = beg; i <= end; i++)
         selectItem(i, sel);
@@ -798,14 +726,10 @@ void YListBox::setFocusedItem(int item, bool clear, bool extend, bool virt) {
     //msg("%d (%d-%d=%d): clear:%d extend:%d virt:%d",
     //       item, fSelectStart, fSelectEnd, fSelect, clear, extend, virt);
 
-    if (virt)
-        fDragging = true;
-    else {
-        fDragging = false;
-    }
+    fDragging = virt;
     bool sel = true;
     if (oldItem != -1 && extend && !clear)
-        sel = isItemSelected(oldItem);
+        sel = isSelected(oldItem);
     if (clear && !extend)
         clearSelection();
     if (item != -1) {
@@ -847,10 +771,7 @@ void YListBox::setFocusedItem(int item, bool clear, bool extend, bool virt) {
 
 bool YListBox::isSelected(int item) {
     YListItem *a = getItem(item);
-
-    if (a == 0)
-        return false;
-    return isSelected(a);
+    return a && isSelected(a);
 }
 
 bool YListBox::isSelected(YListItem *item) { // !!! remove this !!!
@@ -861,13 +782,10 @@ bool YListBox::isSelected(YListItem *item) { // !!! remove this !!!
         return false;
 
     if (fDragging) {
-        int beg = (fSelectStart < fSelectEnd) ? fSelectStart : fSelectEnd;
-        int end = (fSelectStart < fSelectEnd) ? fSelectEnd : fSelectStart;
+        int beg = min(fSelectStart, fSelectEnd);
+        int end = max(fSelectStart, fSelectEnd);
         if (n >= beg && n <= end) {
-            if (fSelect)
-                s = true;
-            else
-                s = false;
+            s = fSelect;
         }
     }
     return s;
