@@ -8,47 +8,43 @@
 #ifndef __YRESOURCEPATH_H
 #define __YRESOURCEPATH_H
 
-#include <stddef.h>
 #include "ypaint.h"
 #include "yarray.h"
 #include "upath.h"
 
-upath findPath(ustring path, int mode, upath name,
-               bool path_relative = false);
-
-struct YPathElement {
-    YPathElement(upath p): fPath(p) {}
-
-    upath joinPath(upath base) const;
-    upath joinPath(upath base, upath name) const;
-
-    upath fPath;
-};
-
 class YResourcePaths: public refcounted {
 public:
-
-    static ref<YResourcePaths> paths();
     static ref<YResourcePaths> subdirs(upath subdir, bool themeOnly = false);
 
-private:
-    YResourcePaths() {}
-    void addDir(upath root, upath rdir, upath sub);
-public:
-    virtual ~YResourcePaths() { }
-
     ref<YPixmap> loadPixmap(upath base, upath name) const;
-///    ref<YPixbuf> loadPixbuf(upath base, upath name, bool const fullAlpha) const;
     ref<YImage> loadImage(upath base, upath name) const;
-    ref<YIcon> loadIcon(upath base, upath name) const;
 
+    static ref<YPixmap> loadPixmapFile(const upath& file);
+    static ref<YImage> loadImageFile(const upath& file);
+
+    void clear() { return fPaths.clear(); }
     int getCount() const { return fPaths.getCount(); }
-    YPathElement *getPath(int index) const { return fPaths[index]; }
+    const upath& getPath(int index) const { return *fPaths[index]; }
+
+    typedef YObjectArray<upath>::IterType IterType;
+    IterType iterator() { return fPaths.iterator(); }
+    IterType reverseIterator() { return fPaths.reverseIterator(); }
+
 protected:
     void verifyPaths(upath base);
-    
+
 private:
-    YArray<YPathElement *> fPaths;
+    YObjectArray<upath> fPaths;
+
+    YResourcePaths() {}
+    void addDir(upath dir);
+
+    template<class Pict>
+    bool loadPict(upath baseName, ref<Pict>* pict) const;
+    template<class Pict>
+    static bool loadPictFile(upath file, ref<Pict>* pict);
 };
 
 #endif
+
+// vim: set sw=4 ts=4 et:
